package Site::SQLFieldBase;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#

use strict;
use vars qw( @ISA %AUTOFIELDS );

use PObject;
@ISA = qw( PObject );

use Site::Page;

%AUTOFIELDS = (

# This contributes to the html field name
			name => undef,
# SQL field this is bound to
			boundto => undef,
# Label on the HTML form to be used
			label => undef,

# Field type
#  text
			type => undef,

# Enabled means the field can be edited in the browser
			enabled => 1,

# Locked means the field will not be part of the update SQL,
# it's still present in the select part.
			locked => 0,

# Visible controls if the field is visible on the browser
			visible => 1,

# If the field is not bound to an SQL field, this should be set
			value => undef,
# Default value for new records.  Overrides value in case of conflict
			default => undef,

# Set by the container object, the container should provide an
# _HTMLNAME function for creating the HTML names
			container => undef,

# If set to false, then it will not be autoescaped for display
			autoescape => 1,

# Sets up for auto-pre/post pend.  Needs to be handled by the container
			href => undef
		);



# For the sql field names, the user class should weed out undef
# vars because they are not relevant to the task at hand.
sub _SQLVALUE
{
	my $Self = shift;
	return( $Self -> value() );
}

sub _SQLSELECT
{
	my $Self = shift;

	return( $Self -> boundto() );
}

sub _SQLUPDATE
{
	my $Self = shift;

	# If enabled and not locked....
	if( ! $Self -> locked() ) 
	{
		return( $Self -> boundto() );
	}

	return( undef );
}

# This really should be overriden by the final class modules
sub _HTMLFIELD
{
	my $Self = shift;
	return( "Class did not override field html" );
}

sub _HTMLLABEL
{
	my $Self = shift;

	if( $Self -> label() )
	{
		return( $Self -> label() );
	}elsif( $Self -> boundto() )
	{
		return( $Self -> boundto() );
	}elsif( $Self -> name() )
	{
		return( $Self -> name() );
	}
}

sub _SETDEFAULT
{
	my $Self = shift;

	$Self -> { value } = $Self -> default();
}

# These are the ACTUAL methods to call by any user classes
sub htmlfield
{
	my $Self = shift;

	my $RVal;

	if( $Self -> visible() )
	{
		if( $Self -> enabled() )
		{
			$RVal = $Self -> _HTMLFIELD();
		}else
		{
			# Not enabled, no editing, just display it
			$RVal = $Self -> _HTMLDISPLAYVALUE();
		}
	}

	# If I'm not locked I must include something
	if( ( ( ! $Self -> locked() ) &&
		( $Self -> visible() ) &&
		( ! $Self -> enabled() ) ) ||
		( ! $Self -> visible() ) )
	{
		# Just a hidden me
		$RVal .= '<INPUT TYPE=HIDDEN NAME=' . 
			$Self -> _HTMLNAME() .
			' VALUE="' .
			$Self -> _HTMLVALUE() .
			'">';
	}

	return( $RVal );
}

sub htmllabel
{
	my $Self = shift;
	if( $Self -> visible() )
	{
		return( $Self -> _HTMLLABEL() );
	}
	return( undef );
}

sub _HTMLVALUE
{
	# Returns the value HTML'ized
	my $Self = shift;
	return( Site::Page -> encode_entities( $Self -> value() ) );
}

# If the value shown is different than the value used,
# then this should be overriden in the subclass
sub _HTMLDISPLAYVALUE
{
	my $Self = shift;
	if( $Self -> autoescape() )
	{
		return( $Self -> _HTMLVALUE() );
	}else
	{
		return( $Self -> value() );
	}
}

sub htmldisplay
{
	my $self = shift;
	if( $self -> visible() )
	{
		return( $self -> _HTMLDISPLAYVALUE() );
	}else
	{
		return( '' );
	}
}

sub _HTMLNAME
{
	# Returns the fully qualified field name
	my $Self = shift;
	if( $Self -> container() )
	{
		return( $Self -> container() -> _HTMLNAME() .
			".fields." .
			$Self -> name() );
	}
	return( $Self -> name() );
}

sub htmlname
{
	return( shift() -> _HTMLNAME() );
}

# This will handle setting my name/binding in the container object,
# if we're contained
sub OBJ_HOOK_name
{
	my $Self = shift;
	my $NewName = shift;
	my $OldName = $Self -> { name };
	my $Fields;

	if( $Self -> container() )
	{
		$Fields = $Self -> container() -> namedfields();
		# Delete our old name
		if( $OldName !~ /^\s*$/ )
		{
			delete( ${ $Fields }{ $OldName } );
		}

		if( $NewName !~ /^\s*$/ )
		{
			#print "Nameing: $NewName\n";
			$Fields -> { $NewName } = $Self;
		}
	}

	return( $NewName );
}

# To delete this field
sub remove
{
	my $Self = shift;
	$Self -> container() -> _DELETEFIELD( $Self );
}

sub OBJ_HOOK_boundto
{
	my $Self = shift;
	my $NewName = shift;
	my $OldName = $Self -> { boundto };
	my $Fields;

	if( $Self -> container() )
	{
		$Fields = $Self -> container() -> boundfields();
		if( $NewName !~ /^\s*$/ )
		{
			$Fields -> { $NewName } = $Self;
		}

		# Delete our old name
		if( $OldName !~ /^\s*$/ )
		{
			delete( ${ $Fields }{ $OldName } );
		}
	}

	if( ! $Self -> name() )
	{
		# Auto-set name too
		$Self -> name( $NewName );
	}

	return( $NewName );
}

sub _SETVALUE
{
	my $Self = shift;

	$Self -> { value } = shift();
}

sub disabled
{
	# Shorthand for locked=1/enabled=0
	my $Self = shift;
	if( shift() )
	{
		$Self -> locked( 1 );
		$Self -> enabled( 0 );
	}else
	{
		$Self -> locked( 0 );
		$Self -> enabled( 1 );
	}
}

sub OBJ_HOOK_href
{
	# If href is set, that means we need to not auto-escape
	my $Self = shift;
	my $Val = shift;

	$Self -> autoescape( 0 );
	return( $Val );
}

1;
